with Ada.Finalization,
   Ada.Unchecked_Deallocation,
   Paskma.Lists;

use Ada.Finalization;

generic
   type Key_Type is private; 
   type Value_Type is private; 
   with function Hash_Function (
         Key : Key_Type ) 
     return Natural; 
   with function "=" (
         Left,            
         Right : Key_Type ) 
     return Boolean; 

package Paskma.Hash_Tables is

   Modulo_100 : constant := 101;
   Modulo_1000 : constant := 1009;
   Modulo_10_000 : constant := 10_007;
   Modulo_100_000 : constant := 100_003;
   Modulo_1_000_000 : constant :=  1_299_721;
   Modulo_10_000_000 : constant :=  10_570_849;


   type Hash_Table is private; 

   procedure Set_Modulo (
         Self   : in out Hash_Table; 
         Modulo :        Positive    ); 

   procedure Insert (
         Self  : in out Hash_Table; 
         Key   :        Key_Type;   
         Value :        Value_Type  ); 

   package Value_Lists is new Paskma.Lists(Value_Type);
   use Value_Lists;
   type Value_List_Accesses is access all Value_Lists.List; 

   Key_Not_Found : exception;

   procedure Get_Values_Access (
         Self   : in     Hash_Table;         
         Key    : in     Key_Type;           
         Values :    out Value_List_Accesses ); 

   package Key_Lists is new Paskma.Lists(Key_Type);
   use Key_Lists;

   procedure Get_Keys (
         Self   : in     Hash_Table;    
         Result : in out Key_Lists.List ); 


private

   type Cell_Type is 
      record 
         Key    : Key_Type;  
         Values : aliased Value_Lists.List;  
      end record; 

   package Cell_Lists is new Paskma.Lists(Cell_Type);
   use Cell_Lists;

   type Cell_List_Access is access Cell_Lists.List; 
   pragma Controlled(Cell_List_Access);
   procedure Free is 
   new Ada.Unchecked_Deallocation(Cell_Lists.List, Cell_List_Access);

   type Cell_Table_Type is array (Natural range <>) of Cell_List_Access; 
   type Cell_Table_Access is access Cell_Table_Type; 
   pragma Controlled(Cell_Table_Access);
   procedure Free is 
   new Ada.Unchecked_Deallocation(Cell_Table_Type, Cell_Table_Access);

   type Hash_Table is new Controlled with 
      record 
         Modulo : Positive          := 1;  
         Table  : Cell_Table_Access;  
      end record; 


   procedure Initialize (
         Self : in out Hash_Table ); 

   procedure Adjust (
         Self : in out Hash_Table ); 

   procedure Finalize (
         Self : in out Hash_Table ); 

end Paskma.Hash_Tables;