with Ada.Finalization;

use Ada.Finalization;

generic
   type Item_Type is private; 

package Paskma.Lists is

   type Item_Access is access all Item_Type; 

   type List is new Controlled with private;
   
   procedure Initialize (
         The_List : in out List ); 
   procedure Finalize (
         The_List : in out List ); 
   procedure Adjust (
         The_List : in out List ); 

   type Iterator is private;
   
   Iterator_Error, List_Empty : exception;

   ------------------------------------------------------------------------------
   function Get_Begin (
         Of_List : in     List ) 
     return Iterator; 
   ------------------------------------------------------------------------------
   function Get_End (
         Of_List : in     List ) 
     return Iterator; 
   ------------------------------------------------------------------------------
   procedure Push_Front (
         Item        : in     Item_Type; 
         To_The_List : in out List       ); 
   ------------------------------------------------------------------------------
   procedure Push_Back (
         Item        : in     Item_Type; 
         To_The_List : in out List       ); 
   ------------------------------------------------------------------------------
   function Length (
         Of_The_List : in     List ) 
     return Natural; 
   ------------------------------------------------------------------------------
   procedure Empty (
         The_List : in out List ); 
   ------------------------------------------------------------------------------
   procedure Remove (
         I         : in out Iterator; 
         From_List : in out List      ); 
   ------------------------------------------------------------------------------
   function "=" (
         Left,                   
         Right : in     Iterator ) 
     return Boolean; 
   ------------------------------------------------------------------------------
   procedure Increment (
         I : in out Iterator ); 
   ------------------------------------------------------------------------------
   procedure Decrement (
         I : in out Iterator ); 
   ------------------------------------------------------------------------------
   function Dereference (
         I : in     Iterator ) 
     return Item_Type; 
   ------------------------------------------------------------------------------
   procedure Dereference (
         I     : in out Iterator; 
         Value : in     Item_Type ); 
   ------------------------------------------------------------------------------
   function Access_Dereference (
         I : in     Iterator ) 
     return Item_Access; 
   ------------------------------------------------------------------------------
private

   type Cell; 
   type Cell_Access is access Cell; 

   pragma Controlled(Cell_Access);

   type Cell is 
      record 
         Value    : aliased Item_Type;  
         Next,  
         Previous : Cell_Access;  
      end record; 

   type List is new Controlled with 
      record 
         Head,  
         Tail  : Cell_Access := null;  
         Count : Natural     := 0;  
      end record; 

   type Iterator is 
      record 
         Actual,  
         Previous : Cell_Access;  
         Is_End   : Boolean     := False;  
      end record; 

end;