with
   Ada.Text_Io,
   Ada.Integer_Text_Io,
   Gnat.Calendar,
   Gnat.Calendar.Time_Io;

use
   Ada.Text_Io,
   Ada.Integer_Text_Io,
   Gnat.Calendar,
   Gnat.Calendar.Time_Io;

package body Paskma.Time_Utilities is

   package Second_Duration_Output is new Ada.Text_Io.Fixed_Io(Second_Duration);
   use Second_Duration_Output;


   procedure Spaces_To_Zeros (
         S : in out String ) is 
   begin
      for I in S'range loop
         if S(I) = ' ' then
            S(I) := '0';
         end if;
      end loop;
   end;

   --
   -- Time_To_String
   --
   -- The output is HH:MM:SS.XX

   function Time_To_String (
         A_Time : Time ) 
     return Unbounded_String is 
      Year    : Year_Number;  
      Month   : Month_Number;  
      Day     : Day_Number;  
      Hour    : Hour_Number;  
      Min     : Minute_Number;  
      Sec     : Second_Number;  
      Sub_Sec : Second_Duration;  

      Result : String (1 .. 11);  
      Tmp    : String (1 .. 10);  
   begin
      Split(A_Time, Year, Month, Day, Hour, Min, Sec, Sub_Sec);

      Put(Result(1..2), Hour);
      Result(3) := ':';

      Put(Result(4..5), Min);
      Result(6) := ':';

      Put(Result(7..8),Sec);

      Put(Tmp, Sub_Sec, 2, 0);
      Result(9..11) := Tmp(Tmp'Last-2..Tmp'Last);

      Spaces_To_Zeros(Result);

      return To_Unbounded_String(Result);
   end;

   --
   -- Date_To_String
   --
   -- The ISO date format is "%Y/%m/%d", e.g. 2002/08/03

   function Date_To_String (
         A_Time : Time ) 
     return Unbounded_String is 
   begin
      return To_Unbounded_String(Image(A_Time, Iso_Date));
   end;

   --
   -- Get_Current_Time
   --

   function Get_Current_Time return Unbounded_String is 
      Now : Time;  
   begin
      Now := Clock;
      return Time_To_String(Now);
   end;

   --
   -- Get_Current_Time
   --

   function Get_Current_Date return Unbounded_String is 
      Now : Time;  
   begin
      Now := Clock;
      return Date_To_String(Now);
   end;

   --
   -- To_Archive_Time_Stamp
   --

   function To_Archive_Time_Stamp (
         From_Time : Time ) 
     return Unbounded_String is 
   begin
      return To_Unbounded_String(Image(From_Time, "%y_%m_%d_%H_%M_%S"));
      -- year, month, day, hour, minute, second - all in two digits format;
      -- there is probably a mistake in the Image function. For my pattern it returns year in four digits...
   end;

   --
   -- Get_Archive_Time_Stamp
   --

   function Get_Archive_Time_Stamp return Unbounded_String is 
   begin
      return To_Archive_Time_Stamp(Clock);
   end;


end Paskma.Time_Utilities;

