with Ada.Characters.Handling,
   Ada.Strings.Fixed;

use Ada.Characters.Handling,
   Ada.Strings.Fixed;

package body Access_Rights is

   -------------
   -- Exclude --
   -------------

   procedure Exclude (
         Obj   : in out Access_Manager_Type; 
         Value :        Access_Right_Type    ) is 
   begin
      Obj.Rights(Value) := False;
   end Exclude;

   -------------
   -- Include --
   -------------

   procedure Include (
         Obj   : in out Access_Manager_Type; 
         Value :        Access_Right_Type    ) is 
   begin
      Obj.Rights(Value) := True;
   end Include;

   ----------------
   -- Initialize --
   ----------------

   procedure Initialize (
         Obj : in out Access_Manager_Type ) is 
   begin
      for I in Access_Right_Type'range loop
         Obj.Rights(I) := False;
      end loop;
   end Initialize;

   -----------------
   -- Is_Included --
   -----------------

   function Is_Included (
         Obj   : in     Access_Manager_Type; 
         Value :        Access_Right_Type    ) 
     return Boolean is 
   begin
      return Obj.Rights(Value);
   end Is_Included;

   ---------
   -- Set --
   ---------

   procedure Set (
         Obj       : in out Access_Manager_Type; 
         With_Data :        Unbounded_String     ) is 

      Upper_Data : constant String := To_Upper (To_String (With_Data));  
   begin

      Initialize(Obj); -- constructo to exclude all rights

      if Index(Upper_Data, "F_R") /= 0 then
         Include(Obj, File_Read);
      end if;

      if Index(Upper_Data, "F_W") /= 0 then
         Include(Obj, File_Write);
      end if;

      if Index(Upper_Data, "F_D") /= 0 then
         Include(Obj, File_Delete);
      end if;

      if Index(Upper_Data, "F_A") /= 0 then
         Include(Obj, File_Append);
      end if;

      if Index(Upper_Data, "D_L") /= 0 then
         Include(Obj, Directory_List);
      end if;

      if Index(Upper_Data, "D_C") /= 0 then
         Include(Obj, Directory_Create);
      end if;

      if Index(Upper_Data, "D_D") /= 0 then
         Include(Obj, Directory_Delete);
      end if;

   end Set;

   --
   --
   --

   function To_String (
         Obj : in     Access_Manager_Type ) 
     return String is 
      Result : String (1 .. 7 * 4) := "****************************";  
   begin
      if Is_Included(Obj, File_Read) then
         Result(1..4) := "F_R ";
      else
         Result(1..4) := "--- ";
      end if;

      if Is_Included(Obj, File_Write) then
         Result(5..8) := "F_W ";
      else
         Result(5..8) := "--- ";
      end if;

      if Is_Included(Obj, File_Append) then
         Result(9..12) := "F_A ";
      else
         Result(9..12) := "--- ";
      end if;

      if Is_Included(Obj, File_Delete) then
         Result(13..16) := "F_D ";
      else
         Result(13..16) := "--- ";
      end if;

      if Is_Included(Obj, Directory_List) then
         Result(17..20) := "D_L ";
      else
         Result(17..20) := "--- ";
      end if;

      if Is_Included(Obj, Directory_Create) then
         Result(21..24) := "D_C ";
      else
         Result(21..24) := "--- ";
      end if;

      if Is_Included(Obj, Directory_Delete) then
         Result(25..28) := "D_D ";
      else
         Result(25..28) := "--- ";
      end if;

      return Result;

   end To_String;

end Access_Rights;

