with Gnat.Sockets,
   Ada.Finalization,
   Ada.Streams.Stream_Io,
   Ftp_Configurations,
   File_Trees,
   Task_Id_Types,
   Statistics,
   Ada.Strings.Unbounded;

use Gnat.Sockets,
   Ada.Finalization,
   Ftp_Configurations,
   File_Trees,
   Task_Id_Types,
   Statistics,
   Ada.Strings.Unbounded;

package Ftp_Protocol_Interpreter is

   --
   -- Keeps the state of protocol interpreter and the state of 
   -- the session
   type Pi_Context_Type is new Controlled with private; 

   procedure Construct (
         Obj                    : in out Pi_Context_Type;          
         Ftp_Configuration      :        Ftp_Configuration_Type;   
         Socket                 :        Socket_Type;              
         Task_Id                :        Task_Id_Type;             
         A_Connected_To_Address :        Sock_Addr_Type;           
         A_Statistics           :        Statistics_Manager_Access ); 

   --
   -- This function is not "real part" the FTP protocol. It is called (from) outside.
   procedure Write_Welcome_Message (
         Obj : Pi_Context_Type ); 

   --
   -- A message consumer called from session task.
   procedure Dispatch_Message (
         Obj     : in out Pi_Context_Type;  
         Message :        Unbounded_String; 
         Quit    :    out Boolean           ); 


private

   --
   -- Is logged any user? Can we accept commands?
   type Log_State_Type is 
         (Logged,  
          Unlogged); 

   --
   -- Some commands are state sensitive. Like Rename from + Rename to
   type Command_State_Type is 
         (Uninteresting,      --neutral state
          Restart_Marker_Set, 
          Waiting_Pass,       
          Waiting_Rnto); 

   type Connection_Mode_Type is 
         (Active, 
          Passive); 

   procedure Initialize (
         Obj : in out Pi_Context_Type ); 


   type Pi_Context_Type is new Controlled with 
      record 
         Log_State       : Log_State_Type;  
         Command_State   : Command_State_Type;  
         Connection_Mode : Connection_Mode_Type;  
         Socket          : Socket_Type;  
         -- Protocol Interpreter channel
         Remote_Address : Sock_Addr_Type;  
         -- data address
         Configuration        : Ftp_Configuration_Type;  
         Task_Id              : Task_Id_Type;  
         File_Tree            : File_Tree_Type;  
         Restart_Marker_Index : Ada.Streams.Stream_Io.Positive_Count;  
         Username,  
         Password             : Unbounded_String;  
         Connected_To_Address : Sock_Addr_Type;  
         -- pi address
         Statistics           : Statistics_Manager_Access;  
         Rename_From_Filename : Unbounded_String;  

         Passive_Listen_Address : Sock_Addr_Type;  
         Passive_Listen_Socket  : Socket_Type;  
      end record; 

end Ftp_Protocol_Interpreter;