with
   Ada.Strings.Unbounded,
   Statistics,
   Ftp_Configurations;

use   Ada.Strings.Unbounded,
   Statistics,
   Ftp_Configurations;

package Ftp_Server is

   procedure Init; 
   procedure Finalize; 

   type Server_Context_Type is limited private; 
   type Parent_Shortcut_Type is private; 


   --
   -- Loads the configuration from given path and starts a new task.
   -- Returns message and success.
   procedure Start (
         Server_Context :    out Server_Context_Type; 
         Config_Path    :        Unbounded_String;    
         Success        :    out Boolean;             
         Message        :    out Unbounded_String     ); 

   --
   -- Stops the task
   procedure Stop (
         Server_Context : in out Server_Context_Type; 
         Message        :    out Unbounded_String     ); 

   function Get_Information (
         Server_Context : Server_Context_Type ) 
     return Unbounded_String; 

   --
   -- Returns true if the task is running
   function Is_Alive (
         Server_Context : Server_Context_Type ) 
     return Boolean; 

private

   --
   -- private types
   --

   type Server_Task_Type; 

   type Server_Task_Access is access Server_Task_Type; 
   pragma Controlled(Server_Task_Access);

   type Server_Context_Type is 
      record 
         Information : aliased Statistics_Manager_Type;  
         Server_Task : Server_Task_Access;  
      end record; 

   type Parent_Shortcut_Type is 
      record 
         Server_Task : Server_Task_Access;  
      end record; 

   --
   -- Server_Task_Type
   --

   task type Server_Task_Type is
      entry Start (
            Load_Config_From             : Ftp_Configuration_Type;    
            Store_Statistics_Information : Statistics_Manager_Access; 
            We_Need_Self                 : Server_Task_Access         ); 
      entry Create_New_Session; 
      entry Stop; 
   end Server_Task_Type;

   --
   -- Private procedure for child package
   --

   procedure Create_New_Session (
         Parent : Parent_Shortcut_Type ); 

end Ftp_Server;