with Socket_Utilities;



with Ada.Exceptions,
   Paskma.Logger;
use Ada.Exceptions,
   Paskma.Logger;

package body Statistics is

   task body Name_Completer_Task_Type is
      The_Statistics : Statistics_Manager_Access;  
      The_Delay      : Duration;  

      Success     : Boolean;  
      Task_Id     : Task_Id_Type;  
      Address     : Sock_Addr_Type;  
      Client_Name : Unbounded_String;  
   begin
      accept Start (
            Statistics : Statistics_Manager_Access; 
            A_Delay    : Duration                   ) do 

         The_Statistics := Statistics;
         The_Delay := A_Delay;
      end Start;

      loop  -- infinite loop with delay
         select
            accept Stop; 
            exit;
         or
            delay The_Delay;
         end select;

         loop -- for all unnamed clients
            begin
               --
               -- try to find unnamed host
               The_Statistics.Get_Unnamed_Ip(Success, Task_Id, Address);

               exit when not Success;

               Log(Task_Id, "Resolving name for: " & Image(Address.Addr));
               --Client_Name := To_Unbounded_String(Official_Name(Get_Host_By_Address(Address.Addr)));
               Client_Name := To_Unbounded_String("gethostbyaddr sux");
               delay 10.0;
               Log(Task_Id, "Name resolved: " & Client_Name & " for [" & Image(Address.Addr) & "]");
               -- and try to get the name
            exception
               when Host_Error =>
                  Client_Name := To_Unbounded_String("unknown");
               when others =>
                  Client_Name := To_Unbounded_String("unexpected error");
            end;

            The_Statistics.Set_Client_Name(Task_Id, Client_Name);

         end loop; -- for all unnamed clients

      end loop; -- infinite loop with delay
   exception
      when Event : others =>
         Log("Unexpected exception in Name_Completer_Task: " & Exception_Information(Event));

   end Name_Completer_Task_Type;


   -----------------------------
   -- Statistics_Manager_Type --
   -----------------------------
   protected body Statistics_Manager_Type is

      --
      -- Image
      --
      -- Private function

      function Image (
            Of_What : Stat_Item_Type ) 
        return Unbounded_String is 
      begin
         return
            Head(Of_What.Username, 10) &
            " id: " & Integer'Image(Integer(Of_What.Task_Id)) &
            " name:  " & Head(Of_What.Name, 23)  & "[" & Image(Of_What.
            Address) & "]" &
            " Actv: " & Of_What.Activity;
      end;

      --------------
      -- Get_Info --
      --------------

      function Get_Info return String_Lists.List is 

         Result :          String_Lists.List;  
         I      :          Stat_Item_Lists.Iterator := Get_Begin (Items);  
         I_End  : constant Stat_Item_Lists.Iterator := Get_End (Items);  
      begin
         while I /= I_End loop
            Push_Back(Image(Dereference(I)), Result);
            Increment(I);
         end loop;

         return Result;

      end Get_Info;

      --
      -- Get_Info
      --

      function Get_Info return Unbounded_String is 

         Result      : Unbounded_String;  
         Semi_Result : String_Lists.List;  
         I,  
         I_End       : String_Lists.Iterator;  
      begin
         Semi_Result := Get_Info;

         I := Get_Begin(Semi_Result);
         I_End := Get_End(Semi_Result);

         while I /= I_End loop
            Result := Result & Dereference(I) & Socket_Utilities.Crlf;
            Increment(I);
         end loop;

         return Result;

      end Get_Info;

      -----------------
      -- Log_In_User --
      -----------------

      procedure Log_In_User (
            Username : Unbounded_String; 
            Task_Id  : Task_Id_Type;     
            Address  : Sock_Addr_Type    ) is 

         New_Item : Stat_Item_Type;  
      begin
         New_Item.Username := Username;
         New_Item.Task_Id := Task_Id;
         New_Item.Address := Address;
         New_Item.Name := Null_Unbounded_String;

         Push_Back(New_Item, Items);
      end Log_In_User;

      --
      -- Get_item
      --
      -- private funciton

      function Get_Item (
            From       : access Stat_Item_Lists.List; 
            Of_Task_Id :        Task_Id_Type          ) 
        return Stat_Item_Lists.Iterator is 

         I : Stat_Item_Lists.Iterator := Get_Begin (From.
         all);
         I_End : constant Stat_Item_Lists.Iterator := Get_End (From.
         all);
      begin

         while I /= I_End loop
            if Dereference(I).Task_Id = Of_Task_Id then
               return I;
            end if;
            Increment(I);
         end loop;

         raise Item_Not_Found;
      end Get_Item;

      ------------------
      -- Log_Out_User --
      ------------------

      procedure Log_Out_User (
            Task_Id : Task_Id_Type ) is 

         To_Logout : Stat_Item_Lists.Iterator;  
      begin
         To_Logout := Get_Item(Items'access, Task_Id);
         Remove(To_Logout, Items);
      end Log_Out_User;

      ------------------
      -- Set_Activity --
      ------------------

      procedure Set_Activity (
            Task_Id  : Task_Id_Type;    
            Activity : Unbounded_String ) is 

         Processed_Item : Stat_Item_Lists.Iterator;  
      begin
         Processed_Item := Get_Item(Items'access, Task_Id);
         Access_Dereference(Processed_Item).all.Activity := Activity;
      end Set_Activity;

      --
      -- Gen_unnamed_ip
      --

      procedure Get_Unnamed_Ip (
            Success :    out Boolean;       
            Task_Id :    out Task_Id_Type;  
            Address :    out Sock_Addr_Type ) is 

         I     :          Stat_Item_Lists.Iterator := Get_Begin (Items);  
         I_End : constant Stat_Item_Lists.Iterator := Get_End (Items);  

      begin
         while I /= I_End loop
            if Dereference(I).Name = Null_Unbounded_String then
               Success := True;
               Task_Id := Dereference(I).Task_Id;
               Address := Dereference(I).Address;
               return;
            end if;
            Increment(I);
         end loop;

         Task_Id := Task_Id_Type'First;
         Success := False;
      end;

      --
      -- Set_Client_Name
      --

      procedure Set_Client_Name (
            Task_Id     : in     Task_Id_Type;    
            Client_Name :        Unbounded_String ) is 
      begin
         Access_Dereference(Get_Item(Items'access, Task_Id)).all.Name :=
            Client_Name;

      exception
         when Item_Not_Found =>
            Log(Task_Id, "Cannot set name, client's task exited.");
      end;

   end Statistics_Manager_Type;

end Statistics;

