unit BridgeU;

interface

uses windows, sysutils;

type TBridge = class
   private
     running : boolean;

     input_read_end,
     input_write_end,
     output_and_error_read_end,
     output_write_end,
     error_write_end,
     log,
     child_process : THandle;

     si : TStartupInfo;
     sa : TsecurityAttributes;
     pi : PROCESS_INFORMATION;
   public
     procedure start(child_name : string);
     procedure send(msg : string);
     procedure read(var element : char);
     procedure WriteToLog(element : char);
     constructor create;
     destructor Destroy; override;
     function isrunning:boolean;
   end;

   TStartingException = class (Exception);
   TIOException = class(exception);

implementation

uses dialogs;

constructor TBridge.Create();
begin
  inherited Create;
  running := false;
end;


function TBridge.IsRunning: boolean;
begin
  result := running;
end;


destructor TBridge.Destroy;
begin
  try
   self.send('exit');
  except
  end;

  //
  // wait until child is ended, maximally 5 sec.
  if WAIT_TIMEOUT = waitForSingleObject(child_process, 5000) then
  begin
    MessageBox(0, 'Cannot shut down child process. Please end it in the Task Manager.', 'xpftp', 0);
    exit;// if the child is not dead, we cannot close the pipes
  end;

  sleep(500);//Let the reader log the final answer

  closeHandle(child_process);
  //
  //close the parent ends of pipes
  closeHandle(output_and_error_read_end);
  closeHandle(input_write_end);

  closeHandle(log);
  inherited destroy;
end;

function GetLastMessage : string;
var
  lpMsgBuf : pointer;
begin
FormatMessage(
            FORMAT_MESSAGE_ALLOCATE_BUFFER or FORMAT_MESSAGE_FROM_SYSTEM,
            nil,
            GetLastError,
            0,//MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
            @lpMsgBuf,
            0,
            nil
        );

        result := pchar(lpmsgbuf);
        
        LocalFree(cardinal(lpmsgbuf));
end;

procedure TBridge.start(child_name : string);
var
  suc : boolean;
  output_and_error_read_end_tmp,
  input_write_end_tmp : THandle;
begin
   fillMemory(@si, sizeof(sa), 0);
   sa.nLength := sizeof(sa);
   sa.bInheritHandle := true;


   suc := CreatePipe(output_and_error_read_end_tmp, output_write_end, @sa, 0);

   // Create a duplicate of the output write handle for the std error
   // write handle. This is necessary in case the child application
   // closes one of its std output handles.
   suc := suc and DuplicateHandle(GetCurrentProcess(), output_write_end,
                   GetCurrentProcess(), @error_write_end,
                   0, true, Duplicate_Same_Access);

   suc := suc and CreatePipe(input_read_end, input_write_end_tmp, @sa, 0);

   // Create new output read handle and the input write handles. Set
   // the Properties to FALSE. Otherwise, the child inherits the
   // properties and, as a result, non-closeable handles to the pipes
   // are created.
   suc := suc and  DuplicateHandle(GetCurrentProcess(), output_and_error_read_end_tmp,
                   GetCurrentProcess(), @output_and_error_read_end,
                   0, false, duplicate_same_access);

   suc := suc and DuplicateHandle(GetCurrentProcess(), input_write_end_tmp,
                   GetCurrentProcess(), @input_write_end,
                   0, false, Duplicate_Same_Access);

   suc := suc and CloseHandle(output_and_error_read_end_tmp);
   suc := suc and CloseHandle(input_write_end_tmp);

   if not suc then raise TStartingException.Create('Error jonglovani s handlama, kurwa.' + GetLastMessage);

   FillMemory(@si, sizeof(si), 0);
   si.hStdInput := input_read_end;
   si.hStdOutput := output_write_end;
   si.hStdError := error_write_end;
   si.dwFlags := STARTF_USESTDHANDLES or STARTF_USESHOWWINDOW;

   suc := CreateProcess(pchar(child_name),//child executable
                 nil,//command line
                 nil,//security attributes
                 nil,//thread security attributes
                 true,//interit handles
                 0,//creation flags
                 nil,//enviroment
                 nil,//current directory
                 si,//startup info
                 pi// out; process information
                );

   running := true;

   if not suc then raise TStartingException.Create('Error creating process. #' + GetLastMessage);


   closehandle(pi.hThread);
   // we don't need this handle

   //
   // Close the child ens of pipes
   closeHandle(input_read_end);
   closeHandle(output_write_end);
   closeHandle(error_write_end);

   child_process := pi.hProcess;

   log := CreateFile('_x_commands.log', GENERIC_WRITE,FILE_SHARE_READ,nil,CREATE_ALWAYS,file_attribute_normal,0);
end;

/////////////////////////////////////////////////////////////////////////////////////

procedure TBridge.Send(msg : string);
var
  written, trash : integer;
  suc : boolean;
  i : integer;
  buffer : array[0..100] of char;
begin
  for i := 1 to length(msg) do
    buffer[i-1] := msg[i];

  buffer[length(msg)] := #13;
  buffer[length(msg)+1] := #10;

  //
  // Write to the log
  suc := WriteFile(log, buffer, length(msg)+2, cardinal(trash), nil);
  if not suc or (trash <> length(msg)+2) then showmessage('error writing log');
  flushfilebuffers(log);

  //
  // Send the message to the child process
  suc := WriteFile(input_write_end, buffer, length(msg)+2, cardinal(written), nil);
  if not suc or (written <> length(msg)+2) then raise TIOException.Create('Write failed. #' + GetLastMessage);

  suc := FlushFileBuffers(input_write_end);
  if not suc then raise TIOException.Create('Error flushing. #' + GetLastMessage);
end;
///////////////////////////////////////////////////////////////////////

procedure TBridge.Read(var element : char);
var
   sent : cardinal;
   suc : boolean;
begin
  suc := readfile(output_and_error_read_end, element, 1, sent, nil);

  if not suc then raise TIOException.Create('Error reading. #' + GetLastMessage);
end;

procedure TBridge.WriteToLog(element : char);
var
  written : cardinal;
  suc : boolean;
begin
  suc := WriteFile(log, element, 1, written, nil);
  if not suc then
    MessageBox(0, pchar(GetLastMessage), 'xpftp', 0);
  

end;

end.

